@@ -45,6 +45,8 @@
 extern Display  *dpy;
 extern Globaldata xobjs;
 extern XCWindowData *areawin;
+extern colorindex *colorlist;
+extern int number_colors;
 
 /*----------------------------------------------------------------------*/
 /* Recursive search for graphic images in an object.			*/
@@ -236,16 +238,31 @@ void UDrawGraphic(graphicptr gp)
     ppt.y -= (gp->target->height >> 1);
 
     if (gp->clipmask != (Pixmap)NULL) {
-       XSetClipOrigin(dpy, areawin->gc, ppt.x, ppt.y);
-       XSetClipMask(dpy, areawin->gc, gp->clipmask);
+       if (areawin->clipped > 0) {
+	  /* Clipmask areawin->clipmask already applied to window. */
+	  /* Modify existing clipmask with the graphic's.	   */
+	  XSetFunction(dpy, areawin->cmgc, GXand);
+	  XCopyArea(dpy, gp->clipmask, areawin->clipmask, areawin->cmgc,
+		0, 0, gp->target->width, gp->target->height, ppt.x, ppt.y);
+          XSetClipMask(dpy, areawin->gc, areawin->clipmask);
+	  XSetFunction(dpy, areawin->cmgc, GXcopy);
+       }
+       else {
+          XSetClipOrigin(dpy, areawin->gc, ppt.x, ppt.y);
+          XSetClipMask(dpy, areawin->gc, gp->clipmask);
+       }
     }
 
     XPutImage(dpy, areawin->window, areawin->gc,
 		gp->target, 0, 0, ppt.x, ppt.y, gp->target->width,
 		gp->target->height);
 
-    if (gp->clipmask != (Pixmap)NULL)
-       XSetClipMask(dpy, areawin->gc, None);
+    if (gp->clipmask != (Pixmap)NULL) {
+       if (areawin->clipped <= 0) {
+	  XSetClipMask(dpy, areawin->gc, None);
+          XSetClipOrigin(dpy, areawin->gc, 0, 0);
+       }
+    }
 }
 
 /*----------------------------------------------------------------------*/
@@ -393,7 +410,6 @@ graphicptr new_graphic(objinstptr destinst, char *filename, int px, int py)
 
     iptr->refcount++;
     NEW_GRAPHIC(gp, destobject);
-    destobject->parts++;
 
     (*gp)->scale = 1.0;
     (*gp)->position.x = px;
@@ -416,6 +432,93 @@ graphicptr new_graphic(objinstptr destinst, char *filename, int px, int py)
     return *gp;
 }
 
+/*----------------------------------------------------------------------*/
+/* Create a gradient field graphic					*/
+/* For now, gradient field is linear white-to-black size 100x100.	*/
+/*----------------------------------------------------------------------*/
+
+graphicptr gradient_field(objinstptr destinst, int px, int py, int c1, int c2)
+{
+    graphicptr *gp;
+    objectptr destobject;
+    objinstptr locdestinst;
+    Imagedata *iptr;
+    int width, height, imax, x, y, i;
+    int r, g, b, rd, gd, bd;
+    char id[11];
+    union {
+       u_char b[4];
+       u_long i;
+    } pixel;
+
+    locdestinst = (destinst == NULL) ? areawin->topinstance : destinst;
+    destobject = locdestinst->thisobject;
+
+    if (c1 < 0) c1 = 0;
+    if (c1 >= number_colors) c1 = 1;
+    if (c2 < 0) c2 = 0;
+    if (c2 >= number_colors) c2 = 1;
+
+    /* Create name "gradientXX"	*/
+
+    y = 0;
+    for (i = 0; i < xobjs.images; i++) {
+       iptr = xobjs.imagelist + i;
+       if (!strncmp(iptr->filename, "gradient", 8)) {
+	  if (sscanf(iptr->filename + 8, "%2d", &x) == 1)
+	     if (x >= y)
+		y = x + 1;
+       }
+    }
+    sprintf(id, "gradient%02d", y);
+
+    width = height = 100;	/* Fixed size, at least for now */
+
+    iptr = addnewimage(id, width, height);
+
+    r = (colorlist[c1].color.red >> 8);
+    g = (colorlist[c1].color.green >> 8);
+    b = (colorlist[c1].color.blue >> 8);
+
+    rd = (colorlist[c2].color.red >> 8) - r;
+    gd = (colorlist[c2].color.green >> 8) - g;
+    bd = (colorlist[c2].color.blue >> 8) - b;
+
+    pixel.b[3] = 0;
+    for (y = 0; y < height; y++) {
+       pixel.b[2] = r + ((y * rd) / (height - 1));
+       pixel.b[1] = g + ((y * gd) / (height - 1));
+       pixel.b[0] = b + ((y * bd) / (height - 1));
+       for (x = 0; x < width; x++) {
+	  XPutPixel(iptr->image, x, y, pixel.i);
+       }
+    }
+
+    iptr->refcount++;
+    NEW_GRAPHIC(gp, destobject);
+
+    (*gp)->scale = 1.0;
+    (*gp)->position.x = px;
+    (*gp)->position.y = py;
+    (*gp)->rotation = 0;
+    (*gp)->color = DEFAULTCOLOR;
+    (*gp)->passed = NULL;
+    (*gp)->clipmask = (Pixmap)NULL;
+    (*gp)->source = iptr->image;
+    (*gp)->target = NULL;
+    (*gp)->trot = 0;
+    (*gp)->tscale = 0;
+
+    calcbboxvalues(locdestinst, (genericptr *)gp);
+    updatepagebounds(destobject);
+    incr_changes(destobject);
+
+    register_for_undo(XCF_Graphic, UNDO_DONE, areawin->topinstance, *gp);
+
+    return *gp;
+}
+
+
 /*----------------------------------------------------------------------*/
 /* Free memory associated with the XImage structure for a graphic.	*/
 /*----------------------------------------------------------------------*/
