@@ -211,6 +211,7 @@ void changetextscale(float newscale)
 
    else if (areawin->selects > 0) {
       float oldscale;
+      Boolean waschanged = FALSE;
       for (osel = areawin->selectlist; osel < areawin->selectlist +
 	     areawin->selects; osel++) {
 	 if (SELECTTYPE(osel) == LABEL) {
@@ -222,15 +223,16 @@ void changetextscale(float newscale)
 	       redrawtext(settext);
 	       register_for_undo(XCF_Rescale, UNDO_MORE, areawin->topinstance,
 			(genericptr)settext, (double)oldscale);
+	       waschanged = TRUE;
 	    }
 	 }
       }
+      if (waschanged) undo_finish_series();
    }
 }
 
 /*--------------------------------------------------------------*/
 /* Auto-scale the drawing to fit the declared page size.	*/
-/* (for now, with fixed 1" margins)				*/
 /*								*/
 /* If the page is declared encapsulated, then do nothing.	*/
 /* If a frame box is on the page, then scale to fit the frame	*/
@@ -242,7 +244,6 @@ void autoscale(int page)
    float newxscale, newyscale;
    float scalefudge = (xobjs.pagelist[page]->coordstyle
 	== CM) ? CMSCALE : INCHSCALE;
-   float margin = 144.0;
    int width, height;
    polyptr framebox;
 
@@ -252,10 +253,10 @@ void autoscale(int page)
       return;
    }
 
-   newxscale = (xobjs.pagelist[page]->pagesize.x - margin)
-	/ scalefudge;
-   newyscale = (xobjs.pagelist[page]->pagesize.y - margin)
-	/ scalefudge;
+   newxscale = (xobjs.pagelist[page]->pagesize.x -
+	(2 * xobjs.pagelist[page]->margins.x)) / scalefudge;
+   newyscale = (xobjs.pagelist[page]->pagesize.y -
+	(2 * xobjs.pagelist[page]->margins.y)) / scalefudge;
 
    if ((framebox = checkforbbox(topobject)) != NULL) {
       int i, minx, miny, maxx, maxy;
@@ -472,6 +473,7 @@ void setdscale(xcWidget w, XPoint *dataptr)
 void setosize(xcWidget w, objinstptr dataptr)
 {
    float tmpres, oldsize;
+   Boolean waschanged = FALSE;
    short *osel;
    objinstptr nsobj;
    int res = sscanf(_STR2, "%f", &tmpres);
@@ -487,12 +489,15 @@ void setosize(xcWidget w, objinstptr dataptr)
 	 oldsize = nsobj->scale;
          nsobj->scale = tmpres;
 
-         if (oldsize != tmpres)
+         if (oldsize != tmpres) {
             register_for_undo(XCF_Rescale, UNDO_MORE, areawin->topinstance,
 			SELTOGENERIC(osel), (double)oldsize);
+	    waschanged = TRUE;
+	 }
       }
    }
-   unselect_all();
+   /* unselect_all(); */
+   if (waschanged) undo_finish_series();
    pwriteback(areawin->topinstance);
    drawarea(NULL, NULL, NULL);
 }
@@ -611,6 +616,7 @@ void setcolorscheme(Boolean boolvalue)
    INFOLABELCOLOR = appdata.infocolor;
    RATSNESTCOLOR = appdata.ratsnestcolor;
    BBOXCOLOR = appdata.bboxpix;
+   CLIPMASKCOLOR = appdata.clipcolor;
 }
 
 /*----------------------------------------------------------------*/
@@ -893,6 +899,8 @@ void setallstylemarks(u_short styleval)
 		(styleval & OPAQUE) ? "opaque" : "transparent");
    XcInternalTagCall(xcinterp, 3, "border", "bbox", (styleval & BBOX) ? "true" :
 		"false");
+   XcInternalTagCall(xcinterp, 3, "border", "clipmask", (styleval & CLIPMASK) ?
+		"true" : "false");
    XcInternalTagCall(xcinterp, 2, "border", (styleval & UNCLOSED) ? "unclosed" :
 		"closed");
    XcInternalTagCall(xcinterp, 2, "border", bptr);
@@ -998,7 +1006,9 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
 	       easydraw(*sstyle, DOFORALL);
 
 #ifdef TCL_WRAPPER
-	       register_for_undo(XCF_ChangeStyle, UNDO_DONE, areawin->topinstance,
+	       register_for_undo(XCF_ChangeStyle,
+			(sstyle == areawin->selectlist + areawin->selects - 1) ?
+			UNDO_DONE : UNDO_MORE, areawin->topinstance,
 			SELTOGENERIC(sstyle), (int)oldstyle);
 #else
 	       /* Tcl version differs in that the element is not deselected after */
@@ -1020,6 +1030,11 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
 	 newstyle &= ~(BBOX);
 	 return -1;
       }
+      else if (value & CLIPMASK) {
+	 Wprintf("Cannot set default style to Clip Mask");
+	 newstyle &= ~(CLIPMASK);
+	 return -1;
+      }
       else {
 	 newstyle &= ~mask;
 	 newstyle |= value;
@@ -1052,7 +1067,7 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
 void setcolor(xcWidget w, int cindex)
 {
    short *scolor;
-   int *ecolor, cval;
+   int *ecolor, cval, oldcolor;
    Boolean selected = False;
    stringpart *strptr, *nextptr;
 
@@ -1085,9 +1100,14 @@ void setcolor(xcWidget w, int cindex)
       for (scolor = areawin->selectlist; scolor < areawin->selectlist
 	   + areawin->selects; scolor++) {
 	 ecolor = &(SELTOCOLOR(scolor));
-
+	 oldcolor = *ecolor;
 	 *ecolor = cval;
 	 selected = True;
+
+	 register_for_undo(XCF_Color, (scolor == areawin->selectlist
+			+ areawin->selects - 1) ?  UNDO_DONE : UNDO_MORE,
+			areawin->topinstance,
+			SELTOGENERIC(scolor), (int)oldcolor);
       }
    }
 
@@ -1096,6 +1116,7 @@ void setcolor(xcWidget w, int cindex)
       if (eventmode != TEXT_MODE && eventmode != ETEXT_MODE)
          areawin->color = cval;      
    }
+   else pwriteback(areawin->topinstance);
 }
 
 /*--------------------------------------------------------------*/
