@@ -67,7 +67,6 @@ extern void FillPolygon(Display *, Window, GC, XPoint *, int, int, int);
 /* from elements.c: */
 
 /* element constructor functions */
-extern int getsubpartindex(pathptr);
 extern labelptr new_label(objinstptr, stringpart *, int, int, int);
 extern labelptr new_simple_label(objinstptr, char *, int, int, int);
 extern labelptr new_temporary_label(objectptr, char *, int, int);
@@ -87,6 +86,7 @@ extern void pathdefaults(pathptr, int, int);
 extern void instancedefaults(objinstptr, objectptr, int, int);
 extern void labeldefaults(labelptr, u_char, int, int);
 
+extern void converttocurve();
 extern void poly_add_point(polyptr, XPoint *);
 extern void drawdot(int, int);
 extern void copyalleparams(genericptr, genericptr);
@@ -97,7 +97,7 @@ extern void charreport(labelptr);
 extern Boolean labeltext(int, char *);
 extern void textreturn(void);
 extern void rejustify(short);
-extern void finddir(polyptr);
+extern void findconstrained(polyptr);
 extern void reversepoints(XPoint *, short);
 extern void reversefpoints(XfPoint *, short);
 extern void freeparts(short *, short);
@@ -106,19 +106,17 @@ extern void unjoin(void);
 extern labelptr findlabelcopy(labelptr, stringpart *);
 extern Boolean neartest(XPoint *, XPoint *);
 extern void join(void);
-extern genericptr getsubpart(pathptr);
+extern genericptr getsubpart(pathptr, int *);
 
 /* interactive manipulation of elements */
 extern void splinebutton(int, int);
-extern void trackspline(xcWidget, caddr_t, caddr_t);
+extern void trackelement(xcWidget, caddr_t, caddr_t);
 extern void arcbutton(int, int);
 extern void trackarc(xcWidget, caddr_t, caddr_t);
 extern void boxbutton(int, int);
 extern void trackbox(xcWidget, caddr_t, caddr_t);
 extern void trackwire(xcWidget, caddr_t, caddr_t);
 extern void startwire(XPoint *);
-extern void trackpoly(xcWidget, caddr_t, caddr_t);
-extern void trackpath(xcWidget, caddr_t, caddr_t);
 extern void setendpoint(short *, short, XPoint **, XPoint *);
 extern void wire_op(int, int ,int);
 
@@ -132,7 +130,7 @@ extern void setpage(Boolean);
 extern int changepage(short);
 extern void newpage(short);
 extern void pushobject(objinstptr);
-extern void push_stack(pushlistptr *, objinstptr);
+extern void push_stack(pushlistptr *, objinstptr, char *);
 extern void pop_stack(pushlistptr *);
 extern void free_stack(pushlistptr *);
 extern void popobject(xcWidget, pointertype, caddr_t);
@@ -152,13 +150,14 @@ extern void zoominrefresh(int, int);
 extern void zoomoutrefresh(int, int);
 extern void panrefresh(u_int, int, int, float);
 extern void checkwarp(XPoint *);
-extern void warparccycle(arcptr);
+extern void warparccycle(arcptr, short);
 extern int checkcycle(genericptr, short);
+extern pointselect *getrefpoint(genericptr, XPoint **);
 extern void copyvirtual(void);
 extern void nextpathcycle(pathptr, short);
-extern void nextpolycycle(polyptr, short);
-extern void nextsplinecycle(splineptr, short);
-extern void nextarccycle(arcptr, short);
+extern void nextpolycycle(polyptr *, short);
+extern void nextsplinecycle(splineptr *, short);
+extern void nextarccycle(arcptr *, short);
 extern void buttonhandler(xcWidget, caddr_t, XButtonEvent *);
 extern void keyhandler(xcWidget, caddr_t, XKeyEvent *);
 extern Boolean compatible_function(int);
@@ -181,8 +180,10 @@ extern void elemrotate(genericptr *, short, XPoint *);
 extern void elementrotate(short, XPoint *);
 extern void edit(int, int);
 extern void pathedit(genericptr);
-extern void xc_lower(short *);
-extern void xc_raise(short *);
+extern void xc_lower();
+extern void xc_raise();
+extern void xc_top(short *, short *);
+extern void xc_bottom(short *, short *);
 extern void exchange(void);
 extern void elhflip(genericptr *, short);
 extern void elvflip(genericptr *, short);
@@ -273,7 +274,9 @@ extern short printparams(FILE *, objinstptr, short);
 extern void printobjectparams(FILE *, objectptr);
 extern void varcheck(FILE *, short, objectptr, short *, genericptr, u_char);
 extern void varfcheck(FILE *, float, objectptr, short *, genericptr, u_char);
-extern void varpcheck(FILE *, short, objectptr, int, short *, genericptr, u_char);
+extern Boolean varpcheck(FILE *, short, objectptr, int, short *, genericptr, u_char);
+extern Boolean varpathcheck(FILE *, short, objectptr, int, short *,
+		genericptr *, pathptr, u_char);
 extern void getfile(xcWidget, pointertype, caddr_t);
 extern int filecmp(char *, char *);
 extern Boolean nextfilename(void);
@@ -297,8 +300,7 @@ extern char *varpscan(objectptr, char *, short *, genericptr, int, int, u_char);
 extern char *varscan(objectptr, char *, short *, genericptr, u_char);
 extern char *varfscan(objectptr, char *, float *, genericptr, u_char);
 extern objinstptr addtoinstlist(int, objectptr, Boolean);
-extern Boolean objectread(FILE *, objectptr, short, short, short, char *,
-		int, TechPtr);
+extern Boolean objectread(FILE *, objectptr, short, short, short, char *, int, TechPtr);
 void importfromlibrary(short, char *, char *);
 objectptr *new_library_object(short, char *, objlistptr *, TechPtr);
 Boolean library_object_unique(short, objectptr, objlistptr);
@@ -442,12 +444,11 @@ extern float UDrawRescaleBox(XPoint *);
 extern void UDrawBBox(void);
 extern void strokepath(XPoint *, short, short, float);
 extern void makesplinepath(splineptr, XPoint *);
-extern void UDrawSpline(splineptr);
-extern void UDrawEditSpline(splineptr);
-extern void UDrawPolygon(polyptr);
-extern void UDrawArc(arcptr);
-extern void UDrawPath(pathptr);
-extern void UDrawObject(objinstptr, short, int, pushlistptr *);
+extern void UDrawSpline(splineptr, float);
+extern void UDrawPolygon(polyptr, float);
+extern void UDrawArc(arcptr, float);
+extern void UDrawPath(pathptr, float);
+extern void UDrawObject(objinstptr, short, int, float, pushlistptr *);
 extern void TopDoLatex(void);
 
 /* from help.c: */
@@ -808,6 +809,7 @@ extern short readcommand(short, FILE *);
 extern void UDrawGraphic(graphicptr);
 extern Imagedata *addnewimage(char *, int, int);
 extern graphicptr new_graphic(objinstptr, char *, int, int);
+extern graphicptr gradient_field(objinstptr, int, int, int, int);
 extern void invalidate_graphics(objectptr);
 extern void freegraphic(graphicptr);
 extern short *collect_graphics(short *);
@@ -880,6 +882,7 @@ extern void enable_selects(objectptr, short *, int);
 extern void disable_selects(objectptr, short *, int);
 extern void selectfilter(xcWidget, pointertype, caddr_t);
 extern Boolean checkselect(short);
+extern Boolean checkselect_draw(short, Boolean);
 extern void geneasydraw(short, int, objectptr, objinstptr);
 extern void gendrawselected(short *, objectptr, objinstptr);
 extern selection *genselectelement(short, u_char, objectptr, objinstptr);
@@ -887,8 +890,8 @@ extern short *allocselect(void);
 extern void setoptionmenu(void);
 extern int test_insideness(int, int, XPoint *);
 extern Boolean pathselect(genericptr *, short, float);
-extern Boolean areaelement(genericptr *);
-extern void selectarea(void);
+extern Boolean areaelement(genericptr *, XPoint *, Boolean, short);
+extern Boolean selectarea(objectptr, XPoint *, short);
 extern void startdesel(xcWidget, caddr_t, caddr_t);
 extern void deselect(xcWidget, caddr_t, caddr_t);
 extern void draw_normal_selected(objectptr, objinstptr);
@@ -898,12 +901,20 @@ extern void clearselects_noundo(void);
 extern void clearselects(void);
 extern void unselect_all(void);
 extern void select_connected_pins();
-extern void reset_connected_pins();
+extern void reset_cycles();
 extern selection *recurselect(short, u_char, pushlistptr *);
 extern short *recurse_select_element(short, u_char);
 extern void startselect(void);
 extern void trackselarea(void);
 extern void trackrescale(void);
+extern pointselect *addcycle(genericptr *, short, u_char);
+extern void addanticycle(pathptr, splineptr, short);
+extern void copycycles(pointselect **, pointselect **);
+extern void advancecycle(genericptr *, short);
+extern void removecycle(genericptr *);
+extern void removeothercycles(pathptr, genericptr);
+extern Boolean checkforcycles(short *, int);
+extern void makerefcycle(pointselect *, short);
 
 /* from text.c: */
 
@@ -928,6 +939,7 @@ extern int textcomp(stringpart *, char *, objinstptr);
 extern int textncomp(stringpart *, char *, objinstptr);
 extern int stringcomp(stringpart *, stringpart *);
 extern Boolean issymbolfont(int);
+extern Boolean issansfont(int);
 extern Boolean isisolatin1(int);
 extern int stringcomprelaxed(stringpart *, stringpart *, objinstptr);
 extern int stringparts(stringpart *);
@@ -942,9 +954,10 @@ extern Boolean pin_is_bus(labelptr, objinstptr);
 extern int find_cardinal(int, labelptr, objinstptr);
 extern int find_ordinal(int, labelptr, objinstptr);
 
-extern short UDrawChar(u_char, short, short, int, int);
+extern short UDrawChar(u_char, short, short, int, int, float);
 extern void UDrawString(labelptr, int, objinstptr);
-extern TextExtents ULength(labelptr, objinstptr, float, short, XPoint *);
+extern void UDrawStringNoX(labelptr, int, objinstptr);
+extern TextExtents ULength(labelptr, objinstptr, short, XPoint *);
 extern void undrawtext(labelptr);
 extern void redrawtext(labelptr);
 extern void composefontlib(short);
