@@ -46,10 +46,11 @@
 /*----------------------------------------------------------------------*/
 
 typedef struct {
-   float angle1;
-   float angle2;
-   short radius;
-   short yaxis;
+   float  angle1;
+   float  angle2;
+   short  radius;
+   short  yaxis;
+   XPoint position;
 } arcinfo;
 
 /* Smaller data structure used when saving paths in the undo stacks.    */
@@ -59,7 +60,6 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;
-   short	cycle;
    u_short	style;
    float	width;
    XPoint	ctrl[4];
@@ -77,6 +77,7 @@ typedef struct editelem {
       pointlist points;		/* original polygon */
       arcinfo *arcspecs;	/* original arc values */
       pathinfo *pathspecs;	/* original parts of a path */
+      XPoint	instpos;	/* original position, for an object instance */ 
    } save;
 } editelement;
 
@@ -158,7 +159,8 @@ short *regen_selection(objinstptr thisinst, uselection *srec)
    Boolean reorder = False;
    short *slist;
 
-   slist = (short *)malloc(srec->number * sizeof(short));
+   if (srec->number > 0)
+      slist = (short *)malloc(srec->number * sizeof(short));
 
    k = 0;
    for (i = 0; i < srec->number; i++) {
@@ -191,13 +193,47 @@ short *regen_selection(objinstptr thisinst, uselection *srec)
    }
 
    if (k == 0) {
-      free(slist);
+      if (srec->number > 0) free(slist);
       return NULL;
    }
    else
       return slist;
 }
 
+/*----------------------------------------------------------------------*/
+/* Use the selection list in the undo record to reorder parts in the	*/
+/* object's part list.  Invert the selection list and return it.	*/
+/*----------------------------------------------------------------------*/
+
+void reorder_selection(Undoptr thisrecord)
+{
+   short *slist, *newlist, snum, i;
+   genericptr *pgen, *plist, egen;
+   objinstptr thisinst = thisrecord->thisinst;
+   objectptr thisobj = thisinst->thisobject;
+
+   snum = (short)thisrecord->idata;
+   slist = (short *)thisrecord->undodata;
+   plist = (genericptr *)malloc(snum * sizeof(genericptr));
+   newlist = (short *)malloc(snum * sizeof(short));
+
+   i = 0;
+   for (pgen = plist; pgen < plist + snum; pgen++) {
+      egen = *(thisobj->plist + i);
+      *(plist + *(slist + i)) = egen;
+      i++;
+   }
+   i = 0;
+   for (pgen = plist; pgen < plist + snum; pgen++) {
+      *(thisobj->plist + i) = *pgen;
+      *(newlist + *(slist + i)) = i;
+      i++;
+   }
+   free(plist);
+   free(thisrecord->undodata);
+   thisrecord->undodata = (char *)newlist;
+}
+
 /*----------------------------------------------------------------------*/
 /* free_editelement ---							*/
 /*									*/
@@ -260,7 +296,7 @@ stringpart *get_original_string(labelptr thislab)
    thisrecord = xobjs.undostack;
    if (thisrecord == NULL) return NULL;
 
-   for (chkrecord = thisrecord->next; chkrecord != NULL; chkrecord = chkrecord->next) {
+   for (chkrecord = thisrecord; chkrecord != NULL; chkrecord = chkrecord->next) {
       switch (chkrecord->type) {
 	 case XCF_Edit:
 	    erec = (editelement *)(chkrecord->undodata);
@@ -287,7 +323,7 @@ int select_previous(Undoptr thisrecord)
    Undoptr chkrecord;
    uselection *srec;
 
-   unselect_all();
+   clearselects_noundo();
    for (chkrecord = thisrecord->next; chkrecord != NULL; chkrecord = chkrecord->next) {
 
       /* Selections may cross page changes, but only if in the same series */
@@ -485,7 +521,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic: case XCF_Join:
 	 /* 1 arg:							*/
 	 /*	egen = element just created (genericptr)		*/
 	 egen = va_arg(args, genericptr);
@@ -516,12 +552,16 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	       erec->save.points =
 			copypoints((pointlist)((splineptr)egen)->ctrl, 4);
 	       break;
+	    case OBJINST:
+	       erec->save.instpos = ((objinstptr)egen)->position;
+	       break;
 	    case ARC:
 	       erec->save.arcspecs = (arcinfo *)malloc(sizeof(arcinfo));
 	       erec->save.arcspecs->angle1 = ((arcptr)egen)->angle1;
 	       erec->save.arcspecs->angle2 = ((arcptr)egen)->angle2;
 	       erec->save.arcspecs->radius = ((arcptr)egen)->radius;
 	       erec->save.arcspecs->yaxis = ((arcptr)egen)->yaxis;
+	       erec->save.arcspecs->position = ((arcptr)egen)->position;
 	       break;
 	    case PATH:
 	       newrecord->idata = ((pathptr)egen)->parts;  /* not needed? */
@@ -550,6 +590,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 
       case XCF_ChangeStyle:
       case XCF_Justify:
+      case XCF_Color:
 	 /* 2 args:							*/
 	 /*	egen = element that was changed (with new value)	*/
 	 /*	oval = old style value					*/
@@ -607,6 +648,15 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	 /* Fprintf(stdout, "Undo: registered move of delta (%d, %d)\n",
 			deltax, deltay); */
 	 break;
+
+       case XCF_Reorder:
+	 /* 2 args:							*/
+	 /*	slist = "before" order of elements			*/
+	 /*	snum = number of elements in list (= # parts)		*/
+	 slist = va_arg(args, short *);
+	 snum = va_arg(args, int);
+	 newrecord->undodata = (char *)slist;
+	 newrecord->idata = snum;
    }
 
    va_end(args);
@@ -637,6 +687,7 @@ short undo_one_action()
    arcptr thisarc;
    splineptr thisspline;
    graphicptr thisgraphic;
+   Boolean need_redraw;
    XCWindowData *savewindow = areawin;
 
    /* Undo the recorded action and shift the undo record pointer.	*/
@@ -736,13 +787,17 @@ short undo_one_action()
 	 /* If there was a previous selection in the undo list,	*/
 	 /* revert to it.					*/
 	 
+	 need_redraw = (areawin->selects > 0) ? True : False;
 	 select_previous(thisrecord);
-	 draw_all_selected();
+	 if (need_redraw)
+	    drawarea(areawin->area, NULL, NULL);
+	 else
+	    draw_all_selected();
 	 break;
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
 		case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-		case XCF_Spline: case XCF_Dot: case XCF_Graphic:
+		case XCF_Spline: case XCF_Dot: case XCF_Graphic: case XCF_Join:
 	 egen = (genericptr)thisrecord->undodata;
 	 i = thisrecord->thisinst->thisobject->parts - 1;
 	 if (egen != *(thisrecord->thisinst->thisobject->plist + i)) {
@@ -780,14 +835,24 @@ short undo_one_action()
 	       tmpinfo.angle2 = earc->angle2; 
 	       tmpinfo.radius = earc->radius; 
 	       tmpinfo.yaxis = earc->yaxis; 
+	       tmpinfo.position = earc->position; 
 	       earc->angle1 = erec->save.arcspecs->angle1;
 	       earc->angle2 = erec->save.arcspecs->angle2;
 	       earc->radius = erec->save.arcspecs->radius;
 	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       earc->position = erec->save.arcspecs->position;
 	       *(erec->save.arcspecs) = tmpinfo;
 	       calcarc(earc);
 	       drawarea(areawin->area, NULL, NULL);
-	    }
+	    } break;
+	    case OBJINST: {
+	       XPoint tmppt;
+	       objinstptr einst = (objinstptr)(erec->element);
+	       tmppt = einst->position;
+	       einst->position = erec->save.instpos;
+	       erec->save.instpos = ((objinstptr)egen)->position;
+	       drawarea(areawin->area, NULL, NULL);
+	    }  break;
 	    case POLYGON: {
 	       pointlist tmppts;
 	       int tmpnum;
@@ -855,6 +920,7 @@ short undo_one_action()
 	 break;
 
       case XCF_Copy:
+	 clearselects_noundo();
 	 srec = (uselection *)thisrecord->undodata;
 	 slist = regen_selection(thisrecord->thisinst, srec);
 	 thisobj = delete_element(thisrecord->thisinst, slist,
@@ -896,6 +962,35 @@ short undo_one_action()
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
+      case XCF_Color:
+	 /* Color changes */
+	 egen = (genericptr)thisrecord->undodata;
+	 snum = thisrecord->idata;
+	 switch(egen->type) {
+	    case PATH:
+	       thispath = (pathptr)(thisrecord->undodata);
+	       thisrecord->idata = thispath->color;
+	       thispath->color = snum;
+	       break;
+	    case POLYGON:
+	       thispoly = (polyptr)(thisrecord->undodata);
+	       thisrecord->idata = thispoly->color;
+	       thispoly->color = snum;
+	       break;
+	    case ARC:
+	       thisarc = (arcptr)(thisrecord->undodata);
+	       thisrecord->idata = thisarc->color;
+	       thisarc->color = snum;
+	       break;
+	    case SPLINE:
+	       thisspline = (splineptr)(thisrecord->undodata);
+	       thisrecord->idata = thisspline->color;
+	       thisspline->color = snum;
+	       break;
+	 }
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
       case XCF_Rescale:
 	 escale = (scaleinfo *)thisrecord->undodata;
 	 egen = escale->element;
@@ -960,7 +1055,12 @@ short undo_one_action()
 	 delta = (XPoint *)thisrecord->undodata;
 	 select_connected_pins();
 	 placeselects(-(delta->x), -(delta->y), NULL);
-	 reset_connected_pins();
+	 reset_cycles();
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
+       case XCF_Reorder:
+	 reorder_selection(thisrecord);
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1139,7 +1239,7 @@ short redo_one_action()
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic: case XCF_Join:
 	 egen = (genericptr)thisrecord->undodata;
 	 undelete_one_element(thisrecord->thisinst, egen);
 	 drawarea(areawin->area, NULL, NULL);
@@ -1163,6 +1263,14 @@ short redo_one_action()
 	       resolveparams(thisrecord->thisinst);
 	       redrawtext(elab);
 	    } break;
+	    case OBJINST: {
+	       XPoint tmppt;
+	       objinstptr einst = (objinstptr)(erec->element);
+	       tmppt = einst->position;
+	       einst->position = erec->save.instpos;
+	       erec->save.instpos = ((objinstptr)egen)->position;
+	       drawarea(areawin->area, NULL, NULL);
+	    }  break;
 	    case ARC: {
 	       arcinfo tmpinfo;
 	       arcptr earc = (arcptr)(erec->element);
@@ -1170,14 +1278,16 @@ short redo_one_action()
 	       tmpinfo.angle2 = earc->angle2; 
 	       tmpinfo.radius = earc->radius; 
 	       tmpinfo.yaxis = earc->yaxis; 
+	       tmpinfo.position = earc->position; 
 	       earc->angle1 = erec->save.arcspecs->angle1;
 	       earc->angle2 = erec->save.arcspecs->angle2;
 	       earc->radius = erec->save.arcspecs->radius;
 	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       earc->position = erec->save.arcspecs->position;
 	       *(erec->save.arcspecs) = tmpinfo;
 	       calcarc(earc);
 	       drawarea(areawin->area, NULL, NULL);
-	    }
+	    } break;
 	    case POLYGON: {
 	       pointlist tmppts;
 	       int tmpnum;
@@ -1274,6 +1384,35 @@ short redo_one_action()
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
+      case XCF_Color:
+	 /* Color changes */
+	 egen = (genericptr)thisrecord->undodata;
+	 snum = thisrecord->idata;
+	 switch(egen->type) {
+	    case PATH:
+	       thispath = (pathptr)(thisrecord->undodata);
+	       thisrecord->idata = thispath->color;
+	       thispath->color = snum;
+	       break;
+	    case POLYGON:
+	       thispoly = (polyptr)(thisrecord->undodata);
+	       thisrecord->idata = thispoly->color;
+	       thispoly->color = snum;
+	       break;
+	    case ARC:
+	       thisarc = (arcptr)(thisrecord->undodata);
+	       thisrecord->idata = thisarc->color;
+	       thisarc->color = snum;
+	       break;
+	    case SPLINE:
+	       thisspline = (splineptr)(thisrecord->undodata);
+	       thisrecord->idata = thisspline->color;
+	       thisspline->color = snum;
+	       break;
+	 }
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
       case XCF_Rescale:
 	 escale = (scaleinfo *)thisrecord->undodata;
 	 egen = escale->element;
@@ -1328,7 +1467,12 @@ short redo_one_action()
 	 delta = (XPoint *)thisrecord->undodata;
 	 select_connected_pins();
 	 placeselects(delta->x, delta->y, NULL);
-	 reset_connected_pins();
+	 reset_cycles();
+	 drawarea(areawin->area, NULL, NULL);
+	 break;
+
+      case XCF_Reorder:
+	 reorder_selection(thisrecord);
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1446,7 +1590,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 
       case XCF_Box: case XCF_Arc: case XCF_Wire: case XCF_Text:
       case XCF_Pin_Label: case XCF_Pin_Global: case XCF_Info_Label:
-      case XCF_Spline: case XCF_Dot: case XCF_Graphic:
+      case XCF_Spline: case XCF_Dot: case XCF_Graphic: case XCF_Join:
 	 /* if MODE_UNDO, the element is on the page, so don't destroy it! */
 	 if (mode == MODE_REDO)
 	    free(thisrecord->undodata);
@@ -1472,6 +1616,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
       case XCF_Push:
       case XCF_ChangeStyle:
       case XCF_Justify:
+      case XCF_Color:
 	 /* Do nothing --- undodata points to a valid element */
 	 break;
 
