@@ -146,8 +146,8 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	(a)->plist = (genericptr *) realloc ((a)->plist, \
 	((a)->parts + 1) * sizeof(genericptr))
 
-#define ENDPART		topobject->plist + topobject->parts
-#define EDITPART	topobject->plist + areawin->editpart
+#define ENDPART		topobject->plist + topobject->parts - 1
+#define EDITPART	topobject->plist + *areawin->selectlist
 
 /* Type checks (argument "a" is type genericptr) */
 
@@ -199,36 +199,43 @@ ERROR: Cannot compile without knowing the size of a pointer.  See xcircuit.h.
 	PLIST_INCR(b); \
 	a = (polyptr *)b->plist + b->parts; \
 	*a = (polyptr) malloc(sizeof(polygon)); \
+	b->parts++; \
 	(*a)->type = POLYGON
 #define NEW_LABEL(a,b) \
 	PLIST_INCR(b); \
 	a = (labelptr *)b->plist + b->parts; \
 	*a = (labelptr) malloc(sizeof(label)); \
+	b->parts++; \
 	(*a)->type = LABEL
 #define NEW_OBJINST(a,b) \
 	PLIST_INCR(b); \
 	a = (objinstptr *)b->plist + b->parts; \
 	*a = (objinstptr) malloc(sizeof(objinst)); \
+	b->parts++; \
 	(*a)->type = OBJINST
 #define NEW_ARC(a,b) \
 	PLIST_INCR(b); \
 	a = (arcptr *)b->plist + b->parts; \
 	*a = (arcptr) malloc(sizeof(arc)); \
+	b->parts++; \
 	(*a)->type = ARC
 #define NEW_SPLINE(a,b) \
 	PLIST_INCR(b); \
 	a = (splineptr *)b->plist + b->parts; \
 	*a = (splineptr) malloc(sizeof(spline)); \
+	b->parts++; \
 	(*a)->type = SPLINE
 #define NEW_PATH(a,b) \
 	PLIST_INCR(b); \
 	a = (pathptr *)b->plist + b->parts; \
 	*a = (pathptr) malloc(sizeof(path)); \
+	b->parts++; \
 	(*a)->type = PATH
 #define NEW_GRAPHIC(a,b) \
 	PLIST_INCR(b); \
 	a = (graphicptr *)b->plist + b->parts; \
 	*a = (graphicptr) malloc(sizeof(graphic)); \
+	b->parts++; \
 	(*a)->type = GRAPHIC
 
 /*----------------------------------------------------------------------*/
@@ -306,6 +313,7 @@ enum editmode {
   COPY_MODE,		/* In the process of copying elements */
   PAN_MODE,		/* In the process of panning to follow the cursor */
   SELAREA_MODE,		/* Area selection box */
+  PENDING_MODE,		/* Temporary mode to select without drawing selection */
   RESCALE_MODE,		/* Interactive element rescaling box */
   CATALOG_MODE,		/* On a library page, library directory, or page directory */
   CATTEXT_MODE,		/* Editing an existing object name in the library */
@@ -421,6 +429,12 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define LIBRARY		3
 #define USERLIB		(xobjs.numlibs + LIBRARY - 1)
 
+/*----------------------------------------------------------------------*/
+/* Object instance styles						*/
+/*----------------------------------------------------------------------*/
+
+#define LINE_INVARIANT 1	/* Linewidth is invariant w.r.t. scale */
+
 /*----------------------------------------------------------------------*/
 /* Box styles								*/
 /*----------------------------------------------------------------------*/
@@ -441,21 +455,34 @@ enum loadmodes {IMPORT = 1, PSBKGROUND, SCRIPT, RECOVER,
 #define OPAQUE 256
 #define BBOX 512
 #define SQUARECAP 1024
+#define CLIPMASK  2048
 
 /*----------------------------------------------------------------------*/
 /* Box edit styles							*/
 /*----------------------------------------------------------------------*/
 
+#define NONE  0
 #define MANHATTAN 1
 #define RHOMBOIDX 2
 #define RHOMBOIDY 4
 #define RHOMBOIDA 8
 
-#define NONE  0
-#define LASTX 1
-#define NEXTX 2
-#define LASTY 4
-#define NEXTY 8
+/*----------------------------------------------------------------------*/
+/* Path edit styles							*/
+/*----------------------------------------------------------------------*/
+
+#define TANGENTS  1		/* (NORMAL = 0) */
+
+/*----------------------------------------------------------------------*/
+/* Cycle points (selected points) (flag bits, can be OR'd together)	*/
+/*----------------------------------------------------------------------*/
+
+#define EDITX 	  0x01
+#define EDITY	  0x02
+#define LASTENTRY 0x04
+#define PROCESS	  0x08
+#define REFERENCE 0x10
+#define ANTIXY	  0x20		/* For matched-tangent curves */
 
 /*----------------------------------------------------------------------*/
 /* Arc creation and edit styles						*/
@@ -651,6 +678,11 @@ typedef struct {
    short *idx;
 } uselection;
 
+typedef struct {
+   short number;
+   u_char flags;
+} pointselect;
+
 /*----------------------------------------------------------------------*/
 /* Bounding box								*/
 /*----------------------------------------------------------------------*/
@@ -670,6 +702,7 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
+   u_short	style;
    XPoint	position;
    short	rotation;
    float	scale;
@@ -695,6 +728,7 @@ typedef struct _pushlist *pushlistptr;
 
 typedef struct _pushlist {
    objinstptr	thisinst;
+   char		*clientdata;	/* general-purpose record */
    pushlistptr	next;
 } pushlist;
 
@@ -737,7 +771,7 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   short	cycle;		/* Edit position, or -1	*/
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    XPoint	position;
    short	rotation;
    float	scale;
@@ -754,9 +788,9 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   short	cycle;		/* Edit position, or -1	*/
    u_short	style;
    float	width;
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    short	number;
    pointlist	points;
 } polygon, *polyptr;
@@ -769,9 +803,9 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   short	cycle;		/* Edit position, or -1	*/
    u_short	style;
    float	width;
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    XPoint	ctrl[4];
    /* the following are for rendering only */
    XfPoint	points[INTSEGS];
@@ -785,9 +819,9 @@ typedef struct {
    u_short	type;
    int		color;
    eparamptr	passed;		/* numerical parameters passed from above */
-   short	cycle;		/* Edit position, or -1	*/
    u_short	style;
    float	width;
+   pointselect	*cycle;		/* Edit position(s), or NULL */
    short	radius; 	/* x-axis radius */
    short	yaxis;		/* y-axis radius */
    float	angle1;		/* endpoint angles, in degrees */
@@ -1120,7 +1154,7 @@ typedef struct {
    Pixel	globalcolor, localcolor, infocolor, ratsnestcolor;
 
    /* non-schematic layout color(s) */
-   Pixel	bboxpix;
+   Pixel	bboxpix, clipcolor;
 
    /* color scheme 1 */
    Pixel	fg, bg;
@@ -1155,9 +1189,12 @@ typedef struct {
 #define XcSetFunction(z) SetFunction(dpy, areawin->gc, z); \
 	areawin->gctype = z
 
-#define XcSetForeground(z) XTopSetForeground(z); areawin->gccolor = \
+#define XcTopSetForeground(z) XTopSetForeground(z); areawin->gccolor = \
 	((z) == DEFAULTCOLOR) ? FOREGROUND : (z)
 
+#define XcSetForeground(z) SetForeground(dpy, areawin->gc, z); \
+	areawin->gccolor = z
+
 #define XcSetXORFg(y,z) XSetXORFg(y,z); areawin->gccolor = \
 	((y) == DEFAULTCOLOR) ? (FOREGROUND ^ z) : (y ^ z)
 
@@ -1275,7 +1312,8 @@ enum {
    XCF_Cancel_Last /* 102 */,	XCF_Sim /* 103 */,
    XCF_SPICE /* 104 */,		XCF_PCB /* 105 */,
    XCF_SPICEflat /* 106 */,	XCF_Rescale /* 107 */,		
-   XCF_Text_Delete_Param /* 108 */,
+   XCF_Reorder /* 108 */,	XCF_Color /* 109 */,
+   XCF_Text_Delete_Param /* 110 */,
    NUM_FUNCTIONS
 };
 
@@ -1298,6 +1336,7 @@ typedef struct {
    short	coordstyle;
    XPoint	drawingscale;
    XPoint	pagesize;	/* size of page to print on */
+   XPoint	margins;
 } Pagedata;
 
 /*----------------------------------------------------------------------*/
@@ -1327,6 +1366,10 @@ typedef struct _windowdata {
    xcWidget	scrollbarh, scrollbarv;
    Window	window;
    GC 		gc;
+   Pixmap	clipmask;
+   Pixmap	pbuf;		/* clipmask buffer for hierarchical clipping */
+   char		clipped;
+   GC		cmgc;
    int		gccolor, gctype;
    xcIntervalId time_id;
 
@@ -1347,6 +1390,7 @@ typedef struct _windowdata {
    short	filter;		/* selection filter */
    Boolean	manhatn;
    Boolean	boxedit;
+   Boolean	pathedit;
    Boolean	snapto;
    Boolean	bboxon;
    Boolean	center;
@@ -1358,6 +1402,7 @@ typedef struct _windowdata {
    Boolean	editinplace;
    Boolean	pinpointon;
    Boolean	pinattach;	/* keep wires attached to pins when moving objinsts */
+   Boolean	showclipmasks;	/* draw clipmask shape outlines */
 #ifndef TCL_WRAPPER
 #ifdef HAVE_XPM
    Boolean	toolbar_on;
@@ -1368,7 +1413,6 @@ typedef struct _windowdata {
    XPoint	save, origin;
    short	selects;
    short	*selectlist;
-   short	editpart;
    short	attachto;
    short	lastlibrary;
    short	textpos;
